<?php #!/usr/local/bin/php 
/**
* Tsuka - FLV Encoder
* @author Jukka Paasonen
* @see http://paazio.nanbudo.fi/
* 
* @lisense The Q Public License Version 1.0
*          http://www.opensource.org/licenses/qtpl.php
*
* @package Tsuka
* @copyright paazio [at] users.sourceforge.net 2005-2007
*
* This file requires atleast PHP version 4.3.2
*/
require('./initialize.php');
header('Content-type: text/plain');
$logdate = 'Y-m-d H:i:s';
// If the encoder binary exists on the system, 
// the value of selected encoder is either ffmpeg or mencoder, 
// then connect to the database.

// $cf['encloglevel']
// 0 = no logging
// 1 = log only main issues
// 2 = debug

if (file_exists($cf['enclocation'])) {
	if (!ini_get('safe_mode') || (ini_get('safe_mode') && preg_match('/'.$cf['enclocation'].'/i', ini_get('safe_mode_exec_dir')))) {
		// At the moment Tsuka supports two server side encoders with two versions:
		// - mencoder 1.0 rc 1 (menc1)
		// - mencoder svn 22834 (mensvn)
		// - ffmpeg 0.4.9-pre1 (ffmpeg0)
		// - ffmpeg svn 8451 (ffmpegsvn).
		if ($cf['encoder'] == 'menc1' || $cf['encoder'] == 'mensvn' || $cf['encoder'] == 'ffmpeg0' || $cf['encoder'] == 'ffmpegsvn') {
			if (!PEAR::isError($mdb2)) {
				/* setting to be fetched:
					 bitrate kbps
					 bitrate variation %
					 audio bitrate kbps
					 audio samplerate hz
					 framerate fps
					 framesize width * height px
					 deinterlacing on or off
					 grayscale video yes/no
				*/
				// Default settings are:
				$enc = array(
					'bitrate' => '300',
					'variation' => '10',
					'framerate' => '15',
					'frameheight' => '260',
					'framewidth' => '400',
					'audiorate' => '56',
					'samplerate' => '22050',
					'deinterlace' => false,
					'grayscale' => false
				);
				print_r($enc);
				$sql = 'SELECT ename, evalue FROM tu_encoder WHERE active=1 LIMIT 100';
				$run = $mdb2->query($sql);
				if (!PEAR::isError($run)) {
					if ($run->numRows() > 0) {
						while ($res = $run->fetchRow(MDB2_FETCHMODE_ASSOC)) {
							$enc[$res['ename']] = $res['evalue'];
							if ($cf['encloglevel'] > 1) {
								$error[] = date($logdate)."\t".'Encoder setting : '.$res['ename'].' = '.$res['evalue'];
							}
						}
					}
					else {
						if ($cf['encloglevel']) {
							$error[] = date($logdate)."\t".'There are no settings available. Using default.';
						}
					}
				}
				else {
					if ($cf['encloglevel']) {
						$error[] = date($logdate)."\t".'Fetching of settings failed. '.$mdb2->getMessage();
					}
				}
				print_r($enc);

				// Contruct the encoding command.
				$bmax = round($enc['bitrate'] * (($enc['variation'] + 100) / 100));
				
				$run1 = '"'.$cf['enclocation'].'" ';

				// Aspect ratio in FLV is always 1:1. Therefore some calculations should be made.
				$width = $enc['framewidth'];
				$height = $enc['frameheight'];

				if ($cf['encoder'] == 'ffmpeg0') {
					$run1 .= '-i ';
					$run2  = ' -b '.$enc['bitrate'].' -maxrate '.$bmax;
					$run2 .= ' -r '.$enc['framerate'].' -bufsize '.($bmax * 2).' -s '.$width.'x'.$height.' -nr 400';
					if ($enc['deinterlace']) {
						$run2 .= ' -deinterlace';
					}
					if ($enc['grayscale']) {
						$run2 .= ' ';
					}
					$run2 .= ' -vtag flv -aic -umv -comment "Tsuka v1.0" -f flv -me epzs -aspect 1:1';
					$run2 .= ' -acodec mp3 -ab '.$enc['audiorate'].' -ar '.$enc['samplerate'].' -ac 1 -y ';
				}
				else if ($cf['encoder'] == 'ffmpegsvn') {
					$run1 .= '-i ';
					$run2  = ' -b '.$enc['bitrate'].'k -maxrate '.$bmax.'k';
					$run2 .= ' -r '.$enc['framerate'].' -bufsize '.($bmax * 2).'k -s 600x344 -nr 400';
					if ($enc['deinterlace']) {
						$run2 .= ' -deinterlace';
					}
					if ($enc['grayscale']) {
						$run2 .= ' ';
					}
					$run2 .= ' -vtag flv -aic -umv -comment "Tsuka v1.0" -f flv -me epzs -aspect 1:1';
					$run2 .= ' -acodec mp3 -ab '.$enc['audiorate'].'k -ar '.$enc['samplerate'].' -ac 1 -y ';
				}
				else if ($cf['encoder'] == 'menc1') {
				}
				else if ($cf['encoder'] == 'mensvn') {
					$run2  = ' -of lavf -ovc lavc -ofps '.$enc['framerate'];
					$run2 .= ' -lavcopts vcodec=flv:vrc_buf_size='.($bmax * 2).':vrc_maxrate='.$bmax;
					$run2 .= ':vbitrate='.$enc['bitrate'].':mbd=2:mv0:trell:v4mv:cbp:last_pred=3:vme=4:dia=2:autoaspect:keyint=15:aic:umv:nr=400';
					$run2 .= ' -oac mp3lame -lameopts cbr:br='.$enc['audiorate'].':mode=3';
					$run2 .= ' -srate '.$enc['samplerate'].' -vf yadif,scale=600:344';
					$run2 .= ' -lavfopts i_certify_that_my_video_stream_does_not_use_b_frames:format=flv';
					$run2 .= ' -o ';
				}
				
				else {
					// This should never happen, so let's put here a trick.
					// Testing for Flash 8 On2 VP6 encoding via binary codec in Mencoder.
					// This will result flipped image, so we need to flip it again.
					
					$run2  = ' -of lavf -ovc vfw -xvfwopts codec=vp6vfw.dll:compdata=on2vp6_260k.mcf -flip';
					//$run2 .= ' -ofps '.$enc['framerate'].' -lavcopts vcodec=flv:vrc_buf_size=600:vrc_maxrate='.$bmax;
					//$run2 .= ':vbitrate='.$enc['bitrate'].':mbd=2:mv0:trell:v4mv:cbp:last_pred=3:vme=4:dia=2:autoaspect:keyint=15';
					$run2 .= ' -oac mp3lame -lameopts cbr:br='.$enc['audiorate'].':mode=3 -srate '.$enc['samplerate'];
					//$run2 .= ' -vf yadif,scale=600:344';
					$run2 .= ' -lavfopts i_certify_that_my_video_stream_does_not_use_b_frames:format=flv';
					$run2 .= ' -o ';
				}

				
				// Encode files.
				$inputfiles = glob($cf['encdirinput'].'*');
				// Iterate trought each file
				foreach ($inputfiles as $infile) {
					// Only continue if the input file is really there.
					if (file_exists($infile) && is_file($infile)) {
						// Create the output file name.
						$tmp = explode(DIRECTORY_SEPARATOR, $infile);
						$tmp = explode('/', end($tmp));
						$infile = str_replace('/', DIRECTORY_SEPARATOR, $infile);
						$outfile = str_replace('/', DIRECTORY_SEPARATOR, $cf['encdiroutput'].strtolower(str_replace(' ', '_', end($tmp))).time().'.flv');
						$doenc = $run1.'"'.$infile.'"'.$run2.'"'.$outfile.'" 2>&1';
						// string exec ( string command [, array &output [, int &return_var]] )
						exec($doenc, $run, $err);
						
						if ($cf['encloglevel'] > 1) {
							$error[] = date($logdate)."\tOutput file is : ".$outfile;
							$error[] = date($logdate)."\tEncoding command is : ".$doenc;
							foreach($run as $rr) {
								$error[] = date($logdate)."\t".$rr;
							}
							$error[] = date($logdate)."\tEncoding return int : ".$err;
						}
						
						if (file_exists($outfile)) {
							// Update the FLV 1.1 metadata so player will actually work.
							$flvupdate = $cf['flvtool'].' -U -creator:"Tsuka v1.0" ';

							if ($cf['encloglevel'] > 1) {
								// Logs FLV stream reading to stream.log in current directory
								$flvupdate .= '-l ';
							}

							$flvupdate .= $outfile.' 2>&1';
							unset($run);
							exec($flvupdate, $run, $err);
							
							if ($cf['encloglevel'] > 1) {
								$error[] = date($logdate)."\tFLVTool2 command is : ".$flvupdate;
								$error[] = date($logdate)."\tFLV update output : ".$err;
							}
		
							// Fix the ownerships and access.
							umask(0000);
							chmod($outfile, 0775);
							chown($outfile, 'paazio');
							chgrp($outfile, 'users');
						}
						else {
							if ($cf['encloglevel']) {
								$error[] = date($logdate)."\tNo output created file : ".$outfile;
							}
						}
					}
					else {
						// No input file existing.
						if ($cf['encloglevel']) {
							$error[] = date($logdate)."\tNo input file : ".$infile;
						}
					}
				}
				$mdb2->disconnect();
			}
			else {
				// Could not connect to the database.
				if ($cf['encloglevel']) {
					$error[] = date($logdate)."\t".'Database connection failed. '.$mdb2->getMessage();
				}
			}
		}
		else {
			if ($cf['encloglevel']) {
				$error[] = date($logdate)."\t".'Encoder not set correctly. Version out of range.';
			}
		}
	}
	else {
		// Safe mode on.
		if ($cf['encloglevel']) {
			$error[] = date($logdate)."\t".'Sorry but since you have safe_mode on, encoder cannot be ran.';
		}
	}
}
else {
	if ($cf['encloglevel']) {
		$error[] = date($logdate)."\t".'Encoder binary ('.$cf['enclocation'].') did not exists.';
	}
}

// Log in to a file.
if ($cf['encloglevel']) {
	$log = fopen($cf['enclog'], 'a');
	// Check for error.
	if (isset($error)) {
		foreach ($error as $err) {
			fwrite($log, $err."\n");
		}
	}
	if ($cf['encloglevel'] > 1) {
		if (function_exists('memory_get_usage')) {
			fwrite($log, date($logdate)."\tMem usage : ".round(memory_get_usage()/1024)." KB\n");
		}
	}
	fclose($log);
}
?>
